@@ -47,8 +47,10 @@ use {
     cryptographic_message_syntax::time_stamp_message_http,
     goblin::mach::{
         constants::{SEG_LINKEDIT, SEG_PAGEZERO, SEG_TEXT},
-        load_command::{CommandVariant, LinkeditDataCommand, SIZEOF_LINKEDIT_DATA_COMMAND},
-        MachO,
+        load_command::{
+            CommandVariant, LinkeditDataCommand, LC_BUILD_VERSION, SIZEOF_LINKEDIT_DATA_COMMAND,
+        },
+        parse_magic_and_ctx, MachO,
     },
     scroll::{IOwrite, Pread},
     std::{borrow::Cow, cmp::Ordering, collections::HashMap, io::Write},
@@ -64,7 +66,8 @@ pub enum CodeSigningSlot {
     ResourceDir,
     Application,
     Entitlements,
-    SecuritySettings,
+    RepSpecific,
+    EntitlementsDer,
     AlternateCodeDirectory0,
     AlternateCodeDirectory1,
     AlternateCodeDirectory2,
@@ -89,8 +92,9 @@ impl std::fmt::Debug for CodeSigningSlot {
             Self::ResourceDir => f.write_fmt(format_args!("Resources ({})", u32::from(*self))),
             Self::Application => f.write_fmt(format_args!("Application ({})", u32::from(*self))),
             Self::Entitlements => f.write_fmt(format_args!("Entitlements ({})", u32::from(*self))),
-            Self::SecuritySettings => {
-                f.write_fmt(format_args!("SecuritySettings ({})", u32::from(*self)))
+            Self::RepSpecific => f.write_fmt(format_args!("Rep Specific ({})", u32::from(*self))),
+            Self::EntitlementsDer => {
+                f.write_fmt(format_args!("DER Entitlements ({})", u32::from(*self)))
             }
             Self::AlternateCodeDirectory0 => f.write_fmt(format_args!(
                 "CodeDirectory Alternate #0 ({})",
@@ -131,7 +135,8 @@ impl From<u32> for CodeSigningSlot {
             3 => Self::ResourceDir,
             4 => Self::Application,
             5 => Self::Entitlements,
-            7 => Self::SecuritySettings,
+            6 => Self::RepSpecific,
+            7 => Self::EntitlementsDer,
             0x1000 => Self::AlternateCodeDirectory0,
             0x1001 => Self::AlternateCodeDirectory1,
             0x1002 => Self::AlternateCodeDirectory2,
@@ -154,7 +159,8 @@ impl From<CodeSigningSlot> for u32 {
             CodeSigningSlot::ResourceDir => 3,
             CodeSigningSlot::Application => 4,
             CodeSigningSlot::Entitlements => 5,
-            CodeSigningSlot::SecuritySettings => 7,
+            CodeSigningSlot::RepSpecific => 6,
+            CodeSigningSlot::EntitlementsDer => 7,
             CodeSigningSlot::AlternateCodeDirectory0 => 0x1000,
             CodeSigningSlot::AlternateCodeDirectory1 => 0x1001,
             CodeSigningSlot::AlternateCodeDirectory2 => 0x1002,
@@ -168,6 +174,13 @@ impl From<CodeSigningSlot> for u32 {
     }
 }
 
+impl CodeSigningSlot {
+    /// Whether this slot has external data (as opposed to provided via a blob).
+    pub fn has_external_content(&self) -> bool {
+        matches!(self, Self::Info | Self::ResourceDir)
+    }
+}
+
 /// Defines header magic for various payloads.
 #[derive(Clone, Copy, Debug, PartialEq, Eq, Hash)]
 pub enum CodeSigningMagic {
@@ -617,6 +630,13 @@ impl<'a> BlobEntry<'a> {
         self.try_into()
     }
 
+    /// Obtain the payload of this blob.
+    ///
+    /// This is the data in the blob without the blob header.
+    pub fn payload(&self) -> Result<&'a [u8], AppleCodesignError> {
+        Ok(read_blob_header(self.data)?.2)
+    }
+
     /// Compute the content digest of this blob using the specified hash type.
     pub fn digest_with(&self, hash: DigestType) -> Result<Vec<u8>, AppleCodesignError> {
         hash.digest(self.data)
@@ -706,6 +726,7 @@ pub enum BlobData<'a> {
     EmbeddedSignature(Box<EmbeddedSignatureBlob<'a>>),
     EmbeddedSignatureOld(Box<EmbeddedSignatureOldBlob<'a>>),
     Entitlements(Box<EntitlementsBlob<'a>>),
+    EntitlementsDer(Box<EntitlementsDerBlob<'a>>),
     DetachedSignature(Box<DetachedSignatureBlob<'a>>),
     BlobWrapper(Box<BlobWrapperBlob<'a>>),
     Other(Box<OtherBlob<'a>>),
@@ -745,6 +766,9 @@ impl<'a> Blob<'a> for BlobData<'a> {
             CodeSigningMagic::Entitlements => {
                 Self::Entitlements(Box::new(EntitlementsBlob::from_blob_bytes(data)?))
             }
+            CodeSigningMagic::EntitlementsDer => {
+                Self::EntitlementsDer(Box::new(EntitlementsDerBlob::from_blob_bytes(data)?))
+            }
             CodeSigningMagic::DetachedSignature => {
                 Self::DetachedSignature(Box::new(DetachedSignatureBlob::from_blob_bytes(data)?))
             }
@@ -763,6 +787,7 @@ impl<'a> Blob<'a> for BlobData<'a> {
             Self::EmbeddedSignature(b) => b.serialize_payload(),
             Self::EmbeddedSignatureOld(b) => b.serialize_payload(),
             Self::Entitlements(b) => b.serialize_payload(),
+            Self::EntitlementsDer(b) => b.serialize_payload(),
             Self::DetachedSignature(b) => b.serialize_payload(),
             Self::BlobWrapper(b) => b.serialize_payload(),
             Self::Other(b) => b.serialize_payload(),
@@ -777,6 +802,7 @@ impl<'a> Blob<'a> for BlobData<'a> {
             Self::EmbeddedSignature(b) => b.to_blob_bytes(),
             Self::EmbeddedSignatureOld(b) => b.to_blob_bytes(),
             Self::Entitlements(b) => b.to_blob_bytes(),
+            Self::EntitlementsDer(b) => b.to_blob_bytes(),
             Self::DetachedSignature(b) => b.to_blob_bytes(),
             Self::BlobWrapper(b) => b.to_blob_bytes(),
             Self::Other(b) => b.to_blob_bytes(),
@@ -1135,6 +1161,42 @@ impl<'a> std::fmt::Display for EntitlementsBlob<'a> {
     }
 }
 
+#[derive(Debug)]
+pub struct EntitlementsDerBlob<'a> {
+    der: Cow<'a, [u8]>,
+}
+
+impl<'a> Blob<'a> for EntitlementsDerBlob<'a> {
+    fn magic() -> u32 {
+        u32::from(CodeSigningMagic::EntitlementsDer)
+    }
+
+    fn from_blob_bytes(data: &'a [u8]) -> Result<Self, AppleCodesignError> {
+        let der = read_and_validate_blob_header(data, Self::magic(), "DER entitlements blob")?;
+
+        Ok(Self { der: der.into() })
+    }
+
+    fn serialize_payload(&self) -> Result<Vec<u8>, AppleCodesignError> {
+        Ok(self.der.to_vec())
+    }
+}
+
+impl<'a> EntitlementsDerBlob<'a> {
+    /// Construct an instance from a [plist::Value].
+    ///
+    /// Not all plists can be encoded to this blob as not all plist value types can
+    /// be encoded to DER. If a plist with an illegal value is passed in, this
+    /// function will error, as DER encoding is performed immediately.
+    ///
+    /// The outermost plist value should be a dictionary.
+    pub fn from_plist(v: &plist::Value) -> Result<Self, AppleCodesignError> {
+        let der = crate::entitlements::der_encode_entitlements_plist(v)?;
+
+        Ok(Self { der: der.into() })
+    }
+}
+
 /// A detached signature.
 #[derive(Debug)]
 pub struct DetachedSignatureBlob<'a> {
@@ -1596,6 +1658,150 @@ pub fn find_signature_data<'a>(
     }
 }
 
+/// Content of an `LC_BUILD_VERSION` load command.
+#[derive(Clone, Debug, Pread)]
+pub struct BuildVersionCommand {
+    /// LC_BUILD_VERSION
+    pub cmd: u32,
+    /// Size of load command data.
+    ///
+    /// sizeof(self) + self.ntools * sizeof(BuildToolsVersion)
+    pub cmdsize: u32,
+    /// Platform identifier.
+    pub platform: u32,
+    /// Minimum operating system version.
+    ///
+    /// X.Y.Z encoded in nibbles as xxxx.yy.zz.
+    pub minos: u32,
+    /// SDK version.
+    ///
+    /// X.Y.Z encoded in nibbles as xxxx.yy.zz.
+    pub sdk: u32,
+    /// Number of tools entries following this structure.
+    pub ntools: u32,
+}
+
+/// Represents `PLATFORM_` mach-o constants.
+pub enum Platform {
+    MacOs,
+    IOs,
+    TvOs,
+    WatchOs,
+    BridgeOs,
+    MacCatalyst,
+    IosSimulator,
+    TvOsSimulator,
+    WatchOsSimulator,
+    DriverKit,
+    Unknown(u32),
+}
+
+impl std::fmt::Display for Platform {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        match self {
+            Self::MacOs => f.write_str("macOS"),
+            Self::IOs => f.write_str("iOS"),
+            Self::TvOs => f.write_str("tvOS"),
+            Self::WatchOs => f.write_str("watchOS"),
+            Self::BridgeOs => f.write_str("bridgeOS"),
+            Self::MacCatalyst => f.write_str("macCatalyst"),
+            Self::IosSimulator => f.write_str("iOSSimulator"),
+            Self::TvOsSimulator => f.write_str("tvOSSimulator"),
+            Self::WatchOsSimulator => f.write_str("watchOSSimulator"),
+            Self::DriverKit => f.write_str("driverKit"),
+            Self::Unknown(v) => f.write_fmt(format_args!("Unknown ({})", v)),
+        }
+    }
+}
+
+impl From<u32> for Platform {
+    fn from(v: u32) -> Self {
+        match v {
+            1 => Self::MacOs,
+            2 => Self::IOs,
+            3 => Self::TvOs,
+            4 => Self::WatchOs,
+            5 => Self::BridgeOs,
+            6 => Self::MacCatalyst,
+            7 => Self::IosSimulator,
+            8 => Self::TvOsSimulator,
+            9 => Self::WatchOsSimulator,
+            10 => Self::DriverKit,
+            _ => Self::Unknown(v),
+        }
+    }
+}
+
+/// Targeting settings for a Mach-O binary.
+pub struct MachoTarget {
+    /// The OS/platform being targeted.
+    pub platform: Platform,
+    /// Minimum required OS version.
+    pub minimum_os_version: semver::Version,
+    /// SDK version targeting.
+    pub sdk_version: semver::Version,
+}
+
+/// Parses and integer with nibbles xxxx.yy.zz into a [semver::Version].
+pub fn parse_version_nibbles(v: u32) -> semver::Version {
+    let major = v >> 16;
+    let minor = v << 16 >> 24;
+    let patch = v & 0xff;
+
+    semver::Version::new(major as _, minor as _, patch as _)
+}
+
+/// Convert a [semver::Version] to a u32 with nibble encoding used by Mach-O.
+pub fn semver_to_macho_target_version(version: &semver::Version) -> u32 {
+    let major = version.major as u32;
+    let minor = version.minor as u32;
+    let patch = version.patch as u32;
+
+    (major << 16) | ((minor & 0xff) << 8) | (patch & 0xff)
+}
+
+/// Attempt to resolve the mach-o targeting settings for a mach-o binary.
+pub fn find_macho_targeting(
+    macho_data: &[u8],
+    macho: &MachO,
+) -> Result<Option<MachoTarget>, AppleCodesignError> {
+    let ctx = parse_magic_and_ctx(macho_data, 0)?
+        .1
+        .expect("context should have been parsed before");
+
+    for lc in &macho.load_commands {
+        if lc.command.cmd() == LC_BUILD_VERSION {
+            let build_version = macho_data.pread_with::<BuildVersionCommand>(lc.offset, ctx.le)?;
+
+            return Ok(Some(MachoTarget {
+                platform: build_version.platform.into(),
+                minimum_os_version: parse_version_nibbles(build_version.minos),
+                sdk_version: parse_version_nibbles(build_version.sdk),
+            }));
+        }
+    }
+
+    for lc in &macho.load_commands {
+        let command = match lc.command {
+            CommandVariant::VersionMinMacosx(c) => Some((c, Platform::MacOs)),
+            CommandVariant::VersionMinIphoneos(c) => Some((c, Platform::IOs)),
+            CommandVariant::VersionMinTvos(c) => Some((c, Platform::TvOs)),
+            CommandVariant::VersionMinWatchos(c) => Some((c, Platform::WatchOs)),
+            _ => None,
+        };
+
+        if let Some((command, platform)) = command {
+            return Ok(Some(MachoTarget {
+                platform,
+                minimum_os_version: parse_version_nibbles(command.version),
+                sdk_version: parse_version_nibbles(command.sdk),
+            }));
+        }
+    }
+
+    Ok(None)
+}
+
 #[cfg(test)]
 mod tests {
     use {
@@ -1763,4 +1969,20 @@ mod tests {
             }
         }
     }
+
+    #[test]
+    fn version_nibbles() {
+        assert_eq!(
+            parse_version_nibbles(12 << 16 | 1 << 8 | 2),
+            semver::Version::new(12, 1, 2)
+        );
+        assert_eq!(
+            parse_version_nibbles(11 << 16 | 10 << 8 | 15),
+            semver::Version::new(11, 10, 15)
+        );
+        assert_eq!(
+            semver_to_macho_target_version(&semver::Version::new(12, 1, 2)),
+            12 << 16 | 1 << 8 | 2
+        );
+    }
 }
