@@ -11,14 +11,17 @@ use {
         code_directory::{CodeDirectoryBlob, CodeSignatureFlags},
         code_hash::compute_code_hashes,
         code_requirement::{CodeRequirementExpression, CodeRequirements},
+        entitlements::plist_to_executable_segment_flags,
         error::AppleCodesignError,
         macho::{
-            create_superblob, AppleSignable, Blob, BlobWrapperBlob, CodeSigningMagic,
+            create_superblob, find_macho_targeting, parse_version_nibbles,
+            semver_to_macho_target_version, AppleSignable, Blob, BlobWrapperBlob, CodeSigningMagic,
             CodeSigningSlot, Digest, DigestType, EmbeddedSignature, EntitlementsBlob,
-            RequirementSetBlob, RequirementType,
+            EntitlementsDerBlob, RequirementSetBlob, RequirementType,
         },
         policy::derive_designated_requirements,
         signing::{DesignatedRequirementMode, SettingsScope, SigningSettings},
+        ExecutableSegmentFlags,
     },
     bcder::{encode::PrimitiveContent, Oid},
     bytes::Bytes,
@@ -31,6 +34,7 @@ use {
         },
         parse_magic_and_ctx, Mach, MachO,
     },
+    log::{info, warn},
     scroll::{ctx::SizeWith, IOwrite},
     std::{borrow::Cow, cmp::Ordering, collections::HashMap, io::Write},
     tugger_apple::create_universal_macho,
@@ -319,6 +323,7 @@ impl<'data> MachOSigner<'data> {
             .iter()
             .enumerate()
             .map(|(index, original_macho)| {
+                info!("signing Mach-O binary at index {}", index);
                 let settings =
                     settings.as_nested_macho_settings(index, original_macho.header.cputype());
 
@@ -339,9 +344,11 @@ impl<'data> MachOSigner<'data> {
 
                 let mut signature_data = self.create_superblob(
                     &settings,
+                    self.macho_data(index),
                     &intermediate_macho,
                     original_macho.code_signature()?.as_ref(),
                 )?;
+                info!("total signature size: {} bytes", signature_data.len());
 
                 // The Mach-O writer adjusts load commands based on the signature length. So pad
                 // with NULLs to get to our placeholder length.
@@ -403,10 +410,13 @@ impl<'data> MachOSigner<'data> {
     pub fn create_superblob(
         &self,
         settings: &SigningSettings,
+        macho_data: &[u8],
         macho: &MachO,
         previous_signature: Option<&EmbeddedSignature>,
     ) -> Result<Vec<u8>, AppleCodesignError> {
-        let code_directory = self.create_code_directory(settings, macho, previous_signature)?;
+        let code_directory =
+            self.create_code_directory(settings, macho_data, macho, previous_signature)?;
+        info!("code directory version: {}", code_directory.version);
 
         // By convention, the Code Directory goes first.
         let mut blobs = vec![(
@@ -445,6 +455,10 @@ impl<'data> MachOSigner<'data> {
             .signing_key()
             .ok_or(AppleCodesignError::NoSigningCertificate)?;
 
+        if let Some(cn) = signing_cert.subject_common_name() {
+            warn!("creating cryptographic signature with certificate {}", cn);
+        }
+
         // We need the blob serialized content of the code directory to compute
         // the message digest using alternate data.
         let code_directory_raw = code_directory.to_blob_bytes()?;
@@ -486,6 +500,7 @@ impl<'data> MachOSigner<'data> {
         };
 
         let signer = if let Some(time_stamp_url) = settings.time_stamp_url() {
+            info!("Using time-stamp server {}", time_stamp_url);
             signer.time_stamp_url(time_stamp_url.clone())?
         } else {
             signer
@@ -535,21 +550,41 @@ impl<'data> MachOSigner<'data> {
     pub fn create_code_directory(
         &self,
         settings: &SigningSettings,
+        macho_data: &[u8],
         macho: &MachO,
         previous_signature: Option<&EmbeddedSignature>,
     ) -> Result<CodeDirectoryBlob<'static>, AppleCodesignError> {
         // TODO support defining or filling in proper values for fields with
         // static values.
 
+        let target = find_macho_targeting(macho_data, macho)?;
+
+        if let Some(target) = &target {
+            info!(
+                "binary targets {} >= {} with SDK {}",
+                target.platform, target.minimum_os_version, target.sdk_version,
+            );
+        }
+
         let previous_cd =
             previous_signature.and_then(|signature| signature.code_directory().unwrap_or(None));
 
         let mut flags = CodeSignatureFlags::empty();
 
         match settings.code_signature_flags(SettingsScope::Main) {
-            Some(additional) => flags |= additional,
+            Some(additional) => {
+                info!(
+                    "adding code signature flags from signing settings: {:?}",
+                    additional
+                );
+                flags |= additional
+            }
             None => {
                 if let Some(previous_cd) = &previous_cd {
+                    info!(
+                        "copying code signature flags from previous code directory: {:?}",
+                        previous_cd.flags
+                    );
                     flags |= previous_cd.flags;
                 }
             }
@@ -557,13 +592,18 @@ impl<'data> MachOSigner<'data> {
 
         // The adhoc flag is set when there is no CMS signature.
         if settings.signing_key().is_none() {
+            info!("creating ad-hoc signature");
             flags |= CodeSignatureFlags::ADHOC;
-        } else {
+        } else if flags.contains(CodeSignatureFlags::ADHOC) {
+            info!("removing ad-hoc code signature flag");
             flags -= CodeSignatureFlags::ADHOC;
         }
 
         // Remove linker signed flag because we're not a linker.
-        flags -= CodeSignatureFlags::LINKER_SIGNED;
+        if flags.contains(CodeSignatureFlags::LINKER_SIGNED) {
+            info!("removing linker signed flag from code signature (we're not a linker)");
+            flags -= CodeSignatureFlags::LINKER_SIGNED;
+        }
 
         // Code limit fields hold the file offset at which code digests stop. This
         // is the file offset in the `__LINKEDIT` segment when the embedded signature
@@ -583,22 +623,90 @@ impl<'data> MachOSigner<'data> {
 
         match settings.executable_segment_flags(SettingsScope::Main) {
             Some(flags) => {
+                info!(
+                    "using executable segment flags from signing settings ({:?})",
+                    flags
+                );
                 exec_seg_flags = Some(flags);
             }
             None => {
                 if let Some(previous_cd) = &previous_cd {
                     if let Some(flags) = previous_cd.exec_seg_flags {
+                        info!(
+                            "using executable segment flags from previous code directory ({:?})",
+                            flags
+                        );
                         exec_seg_flags = Some(flags);
                     }
                 }
             }
         }
 
+        // Entitlements can influence the executable segment flags. So make sure
+        // flags derived from entitlements are always set.
+        if let Some(entitlements) = settings.entitlements_plist(SettingsScope::Main) {
+            let implied_flags = plist_to_executable_segment_flags(entitlements);
+
+            if !implied_flags.is_empty() {
+                info!(
+                    "entitlements imply executable segment flags: {:?}",
+                    implied_flags
+                );
+
+                exec_seg_flags = Some(
+                    exec_seg_flags.unwrap_or_else(ExecutableSegmentFlags::empty) | implied_flags,
+                );
+            }
+        }
+
+        // The runtime version is the SDK version from the targeting loader commands. Same
+        // u32 with nibbles encoding the version.
+        //
+        // If the runtime code signature flag is set, we also need to set the runtime version
+        // or else the activation of the hardened runtime is incomplete.
+
+        // First default to the existing runtime version, if present.
         let runtime = match &previous_cd {
-            Some(previous_cd) => previous_cd.runtime,
+            Some(previous_cd) => {
+                if let Some(version) = previous_cd.runtime {
+                    info!(
+                        "copying hardened runtime version {} from previous code directory",
+                        parse_version_nibbles(version)
+                    );
+                }
+                previous_cd.runtime
+            }
             None => None,
         };
 
+        // If the settings defines a runtime version override, use it.
+        let runtime = match settings.runtime_version(SettingsScope::Main) {
+            Some(version) => {
+                info!(
+                    "using hardened runtime version {} from signing settings",
+                    version
+                );
+                Some(semver_to_macho_target_version(version))
+            }
+            None => runtime,
+        };
+
+        // If we still don't have a runtime but need one, derive from the target SDK.
+        let runtime = if runtime.is_none() && flags.contains(CodeSignatureFlags::RUNTIME) {
+            if let Some(target) = &target {
+                info!(
+                    "using hardened runtime version {} derived from SDK version",
+                    target.sdk_version
+                );
+                Some(semver_to_macho_target_version(&target.sdk_version))
+            } else {
+                warn!("hardened runtime version required but unable to derive suitable version; signature will likely fail Apple checks");
+                None
+            }
+        } else {
+            runtime
+        };
+
         let code_hashes =
             compute_code_hashes(macho, *settings.digest_type(), Some(page_size as usize))?
                 .into_iter()
@@ -618,6 +726,8 @@ impl<'data> MachOSigner<'data> {
             })
             .collect::<Result<HashMap<_, _>, AppleCodesignError>>()?;
 
+        // There is no corresponding blob for the info plist data since it is provided
+        // externally to the embedded signature.
         match settings.info_plist_data(SettingsScope::Main) {
             Some(data) => {
                 special_hashes.insert(
@@ -638,6 +748,8 @@ impl<'data> MachOSigner<'data> {
             }
         }
 
+        // There is no corresponding blob for resources data since it is provided
+        // externally to the embedded signature.
         match settings.code_resources_data(SettingsScope::Main) {
             Some(data) => {
                 special_hashes.insert(
@@ -706,7 +818,7 @@ impl<'data> MachOSigner<'data> {
             special_hashes,
         };
 
-        cd.adjust_version();
+        cd.adjust_version(target);
         cd.clear_newer_fields();
 
         Ok(cd)
@@ -733,6 +845,7 @@ impl<'data> MachOSigner<'data> {
                 // If we are using an Apple-issued cert, this should automatically
                 // derive appropriate designated requirements.
                 if let Some((_, cert)) = settings.signing_key() {
+                    info!("attempting to derive code requirements from signing certificate");
                     let identifier =
                         Some(self.get_binary_identifier(settings, previous_signature)?);
 
@@ -742,6 +855,7 @@ impl<'data> MachOSigner<'data> {
                 }
             }
             DesignatedRequirementMode::Explicit(exprs) => {
+                info!("using provided code requirements");
                 for expr in exprs {
                     requirements.push(CodeRequirementExpression::from_bytes(expr)?.0);
                 }
@@ -749,18 +863,33 @@ impl<'data> MachOSigner<'data> {
         }
 
         if !requirements.is_empty() {
+            info!("code requirements: {}", requirements);
+
             let mut blob = RequirementSetBlob::default();
             requirements.add_to_requirement_set(&mut blob, RequirementType::Designated)?;
 
             res.push((CodeSigningSlot::RequirementSet, blob.to_blob_bytes()?));
         }
 
-        if let Some(entitlements) = settings.entitlements_xml(SettingsScope::Main) {
-            let blob = EntitlementsBlob::from_string(entitlements);
+        if let Some(entitlements) = settings.entitlements_xml(SettingsScope::Main)? {
+            info!("adding entitlements XML");
+            let blob = EntitlementsBlob::from_string(&entitlements);
 
             res.push((CodeSigningSlot::Entitlements, blob.to_blob_bytes()?));
         }
 
+        // The DER encoded entitlements weren't always present in the signature. The feature
+        // appears to have been introduced in macOS 10.14 and is the default behavior as of
+        // macOS 12 "when signing for all platforms." Since `codesign` appears to always add
+        // this blob when entitlements are present, we mimic the behavior. But there may be
+        // scenarios where we want to omit this.
+        if let Some(value) = settings.entitlements_plist(SettingsScope::Main) {
+            info!("adding entitlements DER");
+            let blob = EntitlementsDerBlob::from_plist(value)?;
+
+            res.push((CodeSigningSlot::EntitlementsDer, blob.to_blob_bytes()?));
+        }
+
         Ok(res)
     }
 }
