@@ -2,6 +2,10 @@
 // License, v. 2.0. If a copy of the MPL was not distributed with this
 // file, You can obtain one at https://mozilla.org/MPL/2.0/.
 
+#[allow(unused)]
+mod app_metadata;
+#[allow(unused)]
+mod app_store_connect;
 #[allow(unused)]
 mod apple_certificates;
 #[allow(unused)]
@@ -16,6 +20,8 @@ mod code_hash;
 mod code_requirement;
 #[allow(unused)]
 mod code_resources;
+#[allow(unused)]
+mod entitlements;
 mod error;
 #[allow(unused)]
 mod macho;
@@ -24,6 +30,7 @@ mod macho_signing;
 #[allow(non_upper_case_globals, unused)]
 #[cfg(target_os = "macos")]
 mod macos;
+mod notarization;
 #[allow(unused)]
 mod policy;
 #[allow(unused)]
@@ -31,6 +38,10 @@ mod signing;
 #[allow(unused)]
 mod specification;
 #[allow(unused)]
+mod stapling;
+#[allow(unused)]
+mod ticket_lookup;
+#[allow(unused)]
 mod tutorial;
 #[allow(unused)]
 mod verify;
@@ -47,8 +58,8 @@ use {
         code_requirement::CodeRequirements,
         error::AppleCodesignError,
         macho::{
-            find_signature_data, AppleSignable, Blob, CodeSigningSlot, DigestType,
-            RequirementSetBlob,
+            find_macho_targeting, find_signature_data, AppleSignable, Blob, CodeSigningSlot,
+            DigestType, RequirementSetBlob,
         },
         macho_signing::MachOSigner,
         signing::{SettingsScope, SigningSettings},
@@ -56,7 +67,7 @@ use {
     clap::{Arg, ArgMatches, Command},
     cryptographic_message_syntax::SignedData,
     goblin::mach::{Mach, MachO},
-    slog::{error, o, warn, Drain},
+    log::{error, warn},
     std::{io::Write, path::PathBuf, str::FromStr},
     x509_certificate::{CapturedX509Certificate, EcdsaCurve, InMemorySigningKeyPair, KeyAlgorithm},
 };
@@ -115,6 +126,8 @@ macho-load-commands
    Print information about mach-o load commands in the binary.
 macho-segments
    Print information about mach-o segments in the binary.
+macho-target
+   Print mach-o targeting info (platform and OS/SDK versions).
 requirements-raw
    Raw binary data composing the requirements blob/slot.
 requirements
@@ -158,6 +171,62 @@ certificate pair to a pair of files. The destination files will have
 When the certificate is written to a file, it isn't printed to stdout.
 ";
 
+const NOTARIZE_ABOUT: &str = "\
+Submit a notarization request to Apple.
+
+This command is used to submit an asset to Apple for notarization. Given
+a path to an asset with a code signature, this command will invoke Apple's
+Transporter tool to upload the asset to Apple for them to notarize. *Notarize*
+just means that Apple inspects the uploaded asset and if it validates their
+requirements they will issue a *notarization ticket* that is proof that they
+approved of it. This ticket is then added to the uploaded asset in a process
+called *stapling*.
+
+# Use of Apple Transporter
+
+Upload to Apple requires Apple's Transporter program. This is a Java program
+that does the heavy lifting of sending data to Apple. The program is available
+for Linux, Windows, and macOS and instructions for downloading it can be found
+at https://help.apple.com/itc/transporteruserguide/#/.
+
+This command will look for the executable in the following locations:
+
+1) `APPLE_CODESIGN_TRANSPORTER_EXE` environment variable.
+2) An `iTMSTransporter` executable in PATH
+3) In the default install locations used by Apple's installation docs
+   (e.g. `/usr/local/itms/bin/iTMSTransporter` and
+   `%ProgramFiles%/itms/iTMSTransporter.cmd`).
+
+# App Store Connect API Key
+
+In order to communicate with Apple's servers, you need an App Store Connect
+API Key. This requires an Apple Developer account. You can generate an
+API Key at https://appstoreconnect.apple.com/access/api.
+
+You will need an API Key `AuthKey_<ID>.p8` file on disk in one of the
+default locations used by Apple Transporter. These are
+`$(pwd)/private_keys/`, `~/private_keys/`, `~/.private_keys/`, and
+`~/.appstoreconnect/private_keys/`.
+
+You need to provide both the Key ID and IssuerID when invoking this command.
+Both can be found at https://appstoreconnect.apple.com/access/api.
+
+# Modes of Operation
+
+By default, the `notarize` command will initiate an upload to Apple and exit
+once the upload is complete.
+
+Once an upload is performed, Apple will asynchronously process the uploaded
+content. This can take seconds to minutes.
+
+To poll Apple's servers and wait on the server-side processing to finish,
+specify `--wait`. This will query the state of the processing every few seconds
+until it is finished, the max wait time is reached, or an error occurs.
+
+To automatically staple an asset after server-side processing has finished,
+specify `--staple`. This implies `--wait`.
+";
+
 const PARSE_CODE_SIGNING_REQUIREMENT_ABOUT: &str = "\
 Parse code signing requirement data into human readable text.
 
@@ -340,14 +409,6 @@ const SUPPORTED_HASHES: &[&str; 6] = &[
     "sha512",
 ];
 
-fn get_logger() -> slog::Logger {
-    let decorator = slog_term::TermDecorator::new().build();
-    let drain = slog_term::CompactFormat::new(decorator).build();
-    let drain = std::sync::Mutex::new(drain).fuse();
-
-    slog::Logger::root(drain, o!())
-}
-
 fn parse_scoped_value(s: &str) -> Result<(SettingsScope, &str), AppleCodesignError> {
     let parts = s.splitn(2, ':').collect::<Vec<_>>();
 
@@ -404,9 +465,24 @@ fn add_certificate_source_args(app: Command) -> Command {
     )
 }
 
-fn command_analyze_certificate(args: &ArgMatches) -> Result<(), AppleCodesignError> {
-    let log = get_logger();
+fn add_notarization_upload_args(app: Command) -> Command {
+    app.arg(
+        Arg::new("api_issuer")
+            .long("api-issuer")
+            .takes_value(true)
+            .requires("api_key")
+            .help("App Store Connect Issuer ID (likely a UUID)"),
+    )
+    .arg(
+        Arg::new("api_key")
+            .long("api-key")
+            .takes_value(true)
+            .requires("api_issuer")
+            .help("App Store Connect API Key ID"),
+    )
+}
 
+fn command_analyze_certificate(args: &ArgMatches) -> Result<(), AppleCodesignError> {
     let mut certs = vec![];
 
     if let Some(pfx_path) = args.value_of("pfx_path") {
@@ -421,10 +497,7 @@ fn command_analyze_certificate(args: &ArgMatches) -> Result<(), AppleCodesignErr
                 .expect("should get a single line")
                 .to_string()
         } else {
-            error!(
-                &log,
-                "--pfx-password or --pfx-password-file must be specified"
-            );
+            error!("--pfx-password or --pfx-password-file must be specified");
             return Err(AppleCodesignError::CliBadArgument);
         };
 
@@ -433,7 +506,7 @@ fn command_analyze_certificate(args: &ArgMatches) -> Result<(), AppleCodesignErr
 
     if let Some(values) = args.values_of("der_source") {
         for der_source in values {
-            warn!(&log, "reading DER file {}", der_source);
+            warn!("reading DER file {}", der_source);
             let der_data = std::fs::read(der_source)?;
 
             certs.push(CapturedX509Certificate::from_der(der_data)?);
@@ -442,14 +515,14 @@ fn command_analyze_certificate(args: &ArgMatches) -> Result<(), AppleCodesignErr
 
     if let Some(values) = args.values_of("pem_source") {
         for pem_source in values {
-            warn!(&log, "reading PEM file {}", pem_source);
+            warn!("reading PEM file {}", pem_source);
             let pem_data = std::fs::read(pem_source)?;
 
             for pem in pem::parse_many(&pem_data).map_err(AppleCodesignError::CertificatePem)? {
                 if matches!(pem.tag.as_str(), "CERTIFICATE") {
                     certs.push(CapturedX509Certificate::from_der(pem.contents)?);
                 } else {
-                    warn!(&log, "(unhandled PEM tag {}; ignoring)", pem.tag)
+                    warn!("(unhandled PEM tag {}; ignoring)", pem.tag)
                 }
             }
         }
@@ -927,6 +1000,15 @@ fn command_extract(args: &ArgMatches) -> Result<(), AppleCodesignError> {
                 }
             }
         }
+        "macho-target" => {
+            if let Some(target) = find_macho_targeting(&data, &macho)? {
+                println!("Platform: {}", target.platform);
+                println!("Minimum OS: {}", target.minimum_os_version);
+                println!("SDK: {}", target.sdk_version);
+            } else {
+                println!("Unable to resolve Mach-O targeting from load commands");
+            }
+        }
         "requirements-raw" => {
             let embedded = macho
                 .code_signature()?
@@ -1070,6 +1152,32 @@ fn command_extract(args: &ArgMatches) -> Result<(), AppleCodesignError> {
     Ok(())
 }
 
+fn command_find_transporter() -> Result<(), AppleCodesignError> {
+    if let Some(path) = crate::notarization::find_transporter_exe() {
+        println!("{}", path.display())
+    } else {
+        indoc::eprintdoc! {"
+            Apple Transporter not found.
+
+            This executable is needed to perform notarization.
+
+            Instructions for installing the application are available at
+            https://help.apple.com/itc/transporteruserguide/#/apdAbeb95d60
+
+            We looked in PATH and in common install locations but could not find
+            transporter.
+
+            To force usage of a specific executable, set the {env}
+            environment variable to the path of the executable to use.
+        ",
+            env=crate::notarization::TRANSPORTER_PATH_ENV_VARIABLE
+        };
+        std::process::exit(1);
+    }
+
+    Ok(())
+}
+
 fn command_generate_self_signed_certificate(args: &ArgMatches) -> Result<(), AppleCodesignError> {
     let algorithm = match args
         .value_of("algorithm")
@@ -1192,6 +1300,50 @@ fn command_keychain_export_certificate_chain(_args: &ArgMatches) -> Result<(), A
     ))
 }
 
+fn command_notarize(args: &ArgMatches) -> Result<(), AppleCodesignError> {
+    let path = PathBuf::from(
+        args.value_of("path")
+            .expect("clap should have validated arguments"),
+    );
+    let api_issuer = args.value_of("api_issuer");
+    let api_key = args.value_of("api_key");
+    let staple = args.is_present("staple");
+    let wait = args.is_present("wait") || staple;
+    let max_wait_seconds = args
+        .value_of("max_wait_seconds")
+        .expect("argument should have default value");
+    let max_wait_seconds =
+        u64::from_str(max_wait_seconds).map_err(|_| AppleCodesignError::CliBadArgument)?;
+
+    let wait_duration = std::time::Duration::from_secs(max_wait_seconds);
+
+    let wait_limit = if wait { Some(wait_duration) } else { None };
+
+    let mut notarizer = crate::notarization::Notarizer::new()?;
+
+    if let (Some(issuer), Some(key)) = (api_issuer, api_key) {
+        notarizer.set_api_key(issuer, key)?;
+    }
+
+    let upload = notarizer.notarize_path(&path, wait_limit)?;
+
+    if staple {
+        match upload {
+            crate::notarization::NotarizationUpload::UploadId(_) => {
+                panic!(
+                    "NotarizationUpload::UploadId should not be returned if we waited successfully"
+                );
+            }
+            crate::notarization::NotarizationUpload::DevIdResponse(_) => {
+                let stapler = crate::stapling::Stapler::new()?;
+                stapler.staple_path(&path)?;
+            }
+        }
+    }
+
+    Ok(())
+}
+
 fn command_parse_code_signing_requirement(args: &ArgMatches) -> Result<(), AppleCodesignError> {
     let path = args
         .value_of("input_path")
@@ -1220,8 +1372,6 @@ fn command_parse_code_signing_requirement(args: &ArgMatches) -> Result<(), Apple
 }
 
 fn command_sign(args: &ArgMatches) -> Result<(), AppleCodesignError> {
-    let log = get_logger();
-
     let mut settings = SigningSettings::default();
 
     let mut private_keys = vec![];
@@ -1239,10 +1389,7 @@ fn command_sign(args: &ArgMatches) -> Result<(), AppleCodesignError> {
                 .expect("should get a single line")
                 .to_string()
         } else {
-            error!(
-                &log,
-                "--pfx-password or --pfx-password-file must be specified"
-            );
+            error!("--pfx-password or --pfx-password-file must be specified");
             return Err(AppleCodesignError::CliBadArgument);
         };
 
@@ -1254,7 +1401,7 @@ fn command_sign(args: &ArgMatches) -> Result<(), AppleCodesignError> {
 
     if let Some(values) = args.values_of("pem_source") {
         for pem_source in values {
-            warn!(&log, "reading PEM data from {}", pem_source);
+            warn!("reading PEM data from {}", pem_source);
             let pem_data = std::fs::read(pem_source)?;
 
             for pem in pem::parse_many(&pem_data).map_err(AppleCodesignError::CertificatePem)? {
@@ -1265,14 +1412,14 @@ fn command_sign(args: &ArgMatches) -> Result<(), AppleCodesignError> {
                     "PRIVATE KEY" => {
                         private_keys.push(InMemorySigningKeyPair::from_pkcs8_der(&pem.contents)?)
                     }
-                    tag => warn!(&log, "(unhandled PEM tag {}; ignoring)", tag),
+                    tag => warn!("(unhandled PEM tag {}; ignoring)", tag),
                 }
             }
         }
     }
 
     if private_keys.len() > 1 {
-        error!(&log, "at most 1 PRIVATE KEY can be present; aborting");
+        error!("at most 1 PRIVATE KEY can be present; aborting");
         return Err(AppleCodesignError::CliBadArgument);
     }
 
@@ -1284,21 +1431,17 @@ fn command_sign(args: &ArgMatches) -> Result<(), AppleCodesignError> {
 
     if let Some(signing_key) = &private {
         if public_certificates.is_empty() {
-            error!(
-                &log,
-                "a PRIVATE KEY requires a corresponding CERTIFICATE to pair with it"
-            );
+            error!("a PRIVATE KEY requires a corresponding CERTIFICATE to pair with it");
             return Err(AppleCodesignError::CliBadArgument);
         }
 
         let cert = public_certificates.remove(0);
 
-        warn!(&log, "registering signing key");
+        warn!("registering signing key");
         settings.set_signing_key(signing_key, cert);
         if let Some(certs) = settings.chain_apple_certificates() {
             for cert in certs {
                 warn!(
-                    &log,
                     "automatically registered Apple CA certificate: {}",
                     cert.subject_common_name()
                         .unwrap_or_else(|| "default".into())
@@ -1308,14 +1451,21 @@ fn command_sign(args: &ArgMatches) -> Result<(), AppleCodesignError> {
 
         if let Some(timestamp_url) = args.value_of("timestamp_url") {
             if timestamp_url != "none" {
-                warn!(&log, "using time-stamp protocol server {}", timestamp_url);
+                warn!("using time-stamp protocol server {}", timestamp_url);
                 settings.set_time_stamp_url(timestamp_url)?;
             }
         }
     }
 
+    if let Some(team_id) = settings.set_team_id_from_signing_certificate() {
+        warn!(
+            "automatically setting team ID from signing certificate: {}",
+            team_id
+        );
+    }
+
     for cert in public_certificates {
-        warn!(&log, "registering extra X.509 certificate");
+        warn!("registering extra X.509 certificate");
         settings.chain_certificate(cert);
     }
 
@@ -1343,8 +1493,8 @@ fn command_sign(args: &ArgMatches) -> Result<(), AppleCodesignError> {
             let reqs = CodeRequirements::parse_blob(&code_requirements_data)?.0;
             for expr in reqs.iter() {
                 warn!(
-                    &log,
-                    "setting designated code requirements for {}: {}", scope, expr
+                    "setting designated code requirements for {}: {}",
+                    scope, expr
                 );
                 settings.set_designated_requirement_expression(scope.clone(), expr)?;
             }
@@ -1356,8 +1506,8 @@ fn command_sign(args: &ArgMatches) -> Result<(), AppleCodesignError> {
             let (scope, path) = parse_scoped_value(value)?;
 
             warn!(
-                &log,
-                "setting code resources data for {} from path {}", scope, path
+                "setting code resources data for {} from path {}",
+                scope, path
             );
             let code_resources_data = std::fs::read(path)?;
             settings.set_code_resources_data(scope, code_resources_data);
@@ -1377,12 +1527,9 @@ fn command_sign(args: &ArgMatches) -> Result<(), AppleCodesignError> {
         for value in values {
             let (scope, path) = parse_scoped_value(value)?;
 
-            warn!(
-                &log,
-                "setting entitlments XML for {} from path {}", scope, path
-            );
+            warn!("setting entitlments XML for {} from path {}", scope, path);
             let entitlements_data = std::fs::read_to_string(path)?;
-            settings.set_entitlements_xml(scope, entitlements_data);
+            settings.set_entitlements_xml(scope, entitlements_data)?;
         }
     }
 
@@ -1395,6 +1542,15 @@ fn command_sign(args: &ArgMatches) -> Result<(), AppleCodesignError> {
         }
     }
 
+    if let Some(values) = args.values_of("runtime_version") {
+        for value in values {
+            let (scope, value) = parse_scoped_value(value)?;
+
+            let version = semver::Version::parse(value)?;
+            settings.set_runtime_version(scope, version);
+        }
+    }
+
     if let Some(values) = args.values_of("info_plist_path") {
         for value in values {
             let (scope, value) = parse_scoped_value(value)?;
@@ -1423,7 +1579,7 @@ fn command_sign(args: &ArgMatches) -> Result<(), AppleCodesignError> {
                 })?
                 .to_string_lossy();
 
-            warn!(&log, "setting binary identifier to {}", identifier);
+            warn!("setting binary identifier to {}", identifier);
             settings.set_binary_identifier(SettingsScope::Main, identifier);
         }
 
@@ -1437,26 +1593,37 @@ fn command_sign(args: &ArgMatches) -> Result<(), AppleCodesignError> {
             );
         }
 
-        warn!(&log, "signing {} as a Mach-O binary", input_path.display());
+        warn!("signing {} as a Mach-O binary", input_path.display());
         let macho_data = std::fs::read(input_path)?;
 
-        warn!(&log, "parsing Mach-O");
+        warn!("parsing Mach-O");
         let signer = MachOSigner::new(&macho_data)?;
 
-        warn!(&log, "writing {}", output_path);
+        warn!("writing {}", output_path);
         let mut fh = std::fs::File::create(output_path)?;
         signer.write_signed_binary(&settings, &mut fh)?;
     } else {
-        warn!(&log, "signing {} as a bundle", input_path.display());
+        warn!("signing {} as a bundle", input_path.display());
 
         let signer = BundleSigner::new_from_path(&input_path)?;
 
-        signer.write_signed_bundle(&log, &output_path, &settings)?;
+        signer.write_signed_bundle(&output_path, &settings)?;
     }
 
     Ok(())
 }
 
+fn command_staple(args: &ArgMatches) -> Result<(), AppleCodesignError> {
+    let path = args
+        .value_of("path")
+        .ok_or(AppleCodesignError::CliBadArgument)?;
+
+    let stapler = crate::stapling::Stapler::new()?;
+    stapler.staple_path(path)?;
+
+    Ok(())
+}
+
 fn command_verify(args: &ArgMatches) -> Result<(), AppleCodesignError> {
     let path = args
         .value_of("path")
@@ -1502,6 +1669,8 @@ fn command_x509_oids(_args: &ArgMatches) -> Result<(), AppleCodesignError> {
 }
 
 fn main_impl() -> Result<(), AppleCodesignError> {
+    env_logger::Builder::from_env(env_logger::Env::default().default_filter_or("info")).init();
+
     let app = Command::new("Oxidized Apple Codesigning")
         .version("0.1")
         .author("Gregory Szorc <gregory.szorc@gmail.com>")
@@ -1580,6 +1749,7 @@ fn main_impl() -> Result<(), AppleCodesignError> {
                         "linkedit-segment-raw",
                         "macho-load-commands",
                         "macho-segments",
+                        "macho-target",
                         "requirements-raw",
                         "requirements-rust",
                         "requirements-serialized-raw",
@@ -1600,6 +1770,10 @@ fn main_impl() -> Result<(), AppleCodesignError> {
             ),
     );
 
+    let app = app.subcommand(
+        Command::new("find-transporter").about("Find the path to Apple's transporter executable"),
+    );
+
     let app = app.subcommand(
         Command::new("generate-self-signed-certificate")
             .about("Generate a self-signed certificate for code signing")
@@ -1694,6 +1868,34 @@ fn main_impl() -> Result<(), AppleCodesignError> {
            ),
         );
 
+    let app =
+        app.subcommand(add_notarization_upload_args(
+            Command::new("notarize")
+                .about("Upload an asset to Apple for notarization and possibly staple it")
+                .long_about(NOTARIZE_ABOUT)
+                .arg(
+                    Arg::new("wait")
+                        .long("wait")
+                        .help("Whether to wait for upload processing to complete"),
+                )
+                .arg(
+                    Arg::new("max_wait_seconds")
+                        .long("max-wait-seconds")
+                        .takes_value(true)
+                        .default_value("600")
+                        .help("Maximum time in seconds to wait for the upload result"),
+                )
+                .arg(Arg::new("staple").long("staple").help(
+                    "Staple the notarization ticket after successful upload (implies --wait)",
+                ))
+                .arg(
+                    Arg::new("path")
+                        .takes_value(true)
+                        .required(true)
+                        .help("Path to asset to upload"),
+                ),
+        ));
+
     let app = app.subcommand(
         Command::new("parse-code-signing-requirement")
             .about("Parse binary Code Signing Requirement data into a human readable string")
@@ -1775,6 +1977,11 @@ fn main_impl() -> Result<(), AppleCodesignError> {
                         .takes_value(true)
                         .help("Executable segment flags to set")
                 )
+                .arg(
+                    Arg::new("runtime_version")
+                        .long("runtime-version")
+                        .takes_value(true)
+                        .help("Hardened runtime version to use (defaults to SDK version used to build binary)"))
                 .arg(
                     Arg::new("info_plist_path")
                         .long("info-plist-path")
@@ -1810,6 +2017,16 @@ fn main_impl() -> Result<(), AppleCodesignError> {
                 ),
         ));
 
+    let app = app.subcommand(
+        Command::new("staple")
+            .about("Staples a notarization ticket to an entity")
+            .arg(
+                Arg::new("path")
+                    .required(true)
+                    .help("Path to entity to attempt to staple"),
+            ),
+    );
+
     let app = app.subcommand(
         Command::new("verify")
             .about("Verifies code signature data")
@@ -1831,16 +2048,19 @@ fn main_impl() -> Result<(), AppleCodesignError> {
         Some(("analyze-certificate", args)) => command_analyze_certificate(args),
         Some(("compute-code-hashes", args)) => command_compute_code_hashes(args),
         Some(("extract", args)) => command_extract(args),
+        Some(("find-transporter", _)) => command_find_transporter(),
         Some(("generate-self-signed-certificate", args)) => {
             command_generate_self_signed_certificate(args)
         }
         Some(("keychain-export-certificate-chain", args)) => {
             command_keychain_export_certificate_chain(args)
         }
+        Some(("notarize", args)) => command_notarize(args),
         Some(("parse-code-signing-requirement", args)) => {
             command_parse_code_signing_requirement(args)
         }
         Some(("sign", args)) => command_sign(args),
+        Some(("staple", args)) => command_staple(args),
         Some(("verify", args)) => command_verify(args),
         Some(("x509-oids", args)) => command_x509_oids(args),
         _ => Err(AppleCodesignError::CliUnknownCommand),
