@@ -261,10 +261,11 @@ pub struct SigningSettings<'key> {
     // These are BTreeMap so when we filter the keys, keys with higher precedence come
     // last and last write wins.
     identifiers: BTreeMap<SettingsScope, String>,
-    entitlements: BTreeMap<SettingsScope, String>,
+    entitlements: BTreeMap<SettingsScope, plist::Value>,
     designated_requirement: BTreeMap<SettingsScope, DesignatedRequirementMode>,
     code_signature_flags: BTreeMap<SettingsScope, CodeSignatureFlags>,
     executable_segment_flags: BTreeMap<SettingsScope, ExecutableSegmentFlags>,
+    runtime_version: BTreeMap<SettingsScope, semver::Version>,
     info_plist_data: BTreeMap<SettingsScope, Vec<u8>>,
     code_resources_data: BTreeMap<SettingsScope, Vec<u8>>,
 }
@@ -401,6 +402,35 @@ impl<'key> SigningSettings<'key> {
         self.team_id = Some(value.to_string());
     }
 
+    /// Attempt to set the team ID from the signing certificate.
+    ///
+    /// Apple signing certificates have the team ID embedded within the certificate.
+    /// By calling this method, the team ID embedded within the certificate will
+    /// be propagated to the code signature.
+    ///
+    /// Callers will typically want to call this after registering the signing
+    /// certificate with [Self::set_signing_key()] but before specifying an explicit
+    /// team ID via [Self::set_team_id()].
+    ///
+    /// Calling this will replace a registered team IDs if the signing
+    /// certificate contains a team ID. If no signing certificate is registered or
+    /// it doesn't contain a team ID, no changes will be made.
+    ///
+    /// Returns `Some` if a team ID was set from the signing certificate or `None`
+    /// otherwise.
+    pub fn set_team_id_from_signing_certificate(&mut self) -> Option<&str> {
+        if let Some((_, cert)) = &self.signing_key {
+            if let Some(team_id) = cert.apple_team_id() {
+                self.set_team_id(team_id);
+                Some(self.team_id.as_ref().expect("we just set a team id"))
+            } else {
+                None
+            }
+        } else {
+            None
+        }
+    }
+
     /// Obtain the binary identifier string for a given scope.
     pub fn binary_identifier(&self, scope: impl AsRef<SettingsScope>) -> Option<&str> {
         self.identifiers.get(scope.as_ref()).map(|s| s.as_str())
@@ -419,16 +449,49 @@ impl<'key> SigningSettings<'key> {
         self.identifiers.insert(scope, value.to_string());
     }
 
+    /// Obtain the entitlements plist as a [plist::Value].
+    ///
+    /// The value should be a [plist::Value::Dictionary] variant.
+    pub fn entitlements_plist(&self, scope: impl AsRef<SettingsScope>) -> Option<&plist::Value> {
+        self.entitlements.get(scope.as_ref())
+    }
+
     /// Obtain the entitlements XML string for a given scope.
-    pub fn entitlements_xml(&self, scope: impl AsRef<SettingsScope>) -> Option<&str> {
-        self.entitlements.get(scope.as_ref()).map(|s| s.as_str())
+    pub fn entitlements_xml(
+        &self,
+        scope: impl AsRef<SettingsScope>,
+    ) -> Result<Option<String>, AppleCodesignError> {
+        if let Some(value) = self.entitlements_plist(scope) {
+            let mut buffer = vec![];
+            let writer = std::io::Cursor::new(&mut buffer);
+            value
+                .to_writer_xml(writer)
+                .map_err(AppleCodesignError::PlistSerializeXml)?;
+
+            Ok(Some(
+                String::from_utf8(buffer).expect("plist XML serialization should produce UTF-8"),
+            ))
+        } else {
+            Ok(None)
+        }
     }
 
     /// Set the entitlements to sign via an XML string.
     ///
-    /// The value should be an XML plist. The value is not validated.
-    pub fn set_entitlements_xml(&mut self, scope: SettingsScope, value: impl ToString) {
-        self.entitlements.insert(scope, value.to_string());
+    /// The value should be an XML plist. The value is parsed and stored as
+    /// a native plist value.
+    pub fn set_entitlements_xml(
+        &mut self,
+        scope: SettingsScope,
+        value: impl ToString,
+    ) -> Result<(), AppleCodesignError> {
+        let cursor = std::io::Cursor::new(value.to_string().into_bytes());
+        let value =
+            plist::Value::from_reader_xml(cursor).map_err(AppleCodesignError::PlistParseXml)?;
+
+        self.entitlements.insert(scope, value);
+
+        Ok(())
     }
 
     /// Obtain the designated requirements for a given scope.
@@ -585,6 +648,21 @@ impl<'key> SigningSettings<'key> {
             .map(|x| x.as_slice())
     }
 
+    /// Obtain the runtime version for a given scope.
+    ///
+    /// The runtime version represents an OS version.
+    pub fn runtime_version(&self, scope: impl AsRef<SettingsScope>) -> Option<&semver::Version> {
+        self.runtime_version.get(scope.as_ref())
+    }
+
+    /// Set the runtime version to use in the code directory for a given scope.
+    ///
+    /// The runtime version corresponds to an OS version. The runtime version is usually
+    /// derived from the SDK version used to build the binary.
+    pub fn set_runtime_version(&mut self, scope: SettingsScope, version: semver::Version) {
+        self.runtime_version.insert(scope, version);
+    }
+
     /// Define the `Info.plist` content.
     ///
     /// Signatures can reference the digest of an external `Info.plist` file in
@@ -736,6 +814,12 @@ impl<'key> SigningSettings<'key> {
                 .into_iter()
                 .filter_map(|(key, value)| key_map(key).map(|key| (key, value)))
                 .collect::<BTreeMap<_, _>>(),
+            runtime_version: self
+                .runtime_version
+                .clone()
+                .into_iter()
+                .filter_map(|(key, value)| key_map(key).map(|key| (key, value)))
+                .collect::<BTreeMap<_, _>>(),
             info_plist_data: self
                 .info_plist_data
                 .clone()
@@ -754,7 +838,20 @@ impl<'key> SigningSettings<'key> {
 
 #[cfg(test)]
 mod tests {
-    use super::*;
+    use {super::*, indoc::indoc};
+
+    const ENTITLEMENTS_XML: &str = indoc! {r#"
+        <?xml version="1.0" encoding="UTF-8"?>
+        <!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
+        <plist version="1.0">
+        <dict>
+            <key>application-identifier</key>
+            <string>appid</string>
+            <key>com.apple.developer.team-identifier</key>
+            <string>ABCDEF</string>
+        </dict>
+        </plist>
+    "#};
 
     #[test]
     fn parse_settings_scope() {
@@ -819,10 +916,10 @@ mod tests {
             SettingsScope::MultiArchCpuType(CPU_TYPE_X86_64),
             CodeSignatureFlags::RESTRICT,
         );
-        main_settings.set_entitlements_xml(SettingsScope::MultiArchIndex(0), "index_0");
-        main_settings.set_entitlements_xml(
+        main_settings.set_info_plist_data(SettingsScope::MultiArchIndex(0), b"index_0".to_vec());
+        main_settings.set_info_plist_data(
             SettingsScope::MultiArchCpuType(CPU_TYPE_X86_64),
-            "cpu_x86_64",
+            b"cpu_x86_64".to_vec(),
         );
 
         let macho_settings = main_settings.as_nested_macho_settings(0, CPU_TYPE_ARM64);
@@ -835,8 +932,8 @@ mod tests {
             Some(CodeSignatureFlags::FORCE_HARD)
         );
         assert_eq!(
-            macho_settings.entitlements_xml(SettingsScope::Main),
-            Some("index_0")
+            macho_settings.info_plist_data(SettingsScope::Main),
+            Some(b"index_0".as_ref())
         );
 
         let macho_settings = main_settings.as_nested_macho_settings(0, CPU_TYPE_X86_64);
@@ -849,128 +946,142 @@ mod tests {
             Some(CodeSignatureFlags::RESTRICT)
         );
         assert_eq!(
-            macho_settings.entitlements_xml(SettingsScope::Main),
-            Some("cpu_x86_64")
+            macho_settings.info_plist_data(SettingsScope::Main),
+            Some(b"cpu_x86_64".as_ref())
         );
     }
 
     #[test]
     fn as_bundle_macho_settings() {
         let mut main_settings = SigningSettings::default();
-        main_settings.set_entitlements_xml(SettingsScope::Main, "main");
-        main_settings.set_entitlements_xml(
+        main_settings.set_info_plist_data(SettingsScope::Main, b"main".to_vec());
+        main_settings.set_info_plist_data(
             SettingsScope::Path("Contents/MacOS/main".into()),
-            "main_exe",
+            b"main_exe".to_vec(),
         );
-        main_settings.set_entitlements_xml(
+        main_settings.set_info_plist_data(
             SettingsScope::PathMultiArchIndex("Contents/MacOS/main".into(), 0),
-            "main_exe_index_0",
+            b"main_exe_index_0".to_vec(),
         );
-        main_settings.set_entitlements_xml(
+        main_settings.set_info_plist_data(
             SettingsScope::PathMultiArchCpuType("Contents/MacOS/main".into(), CPU_TYPE_X86_64),
-            "main_exe_x86_64",
+            b"main_exe_x86_64".to_vec(),
         );
 
         let macho_settings = main_settings.as_bundle_macho_settings("Contents/MacOS/main");
         assert_eq!(
-            macho_settings.entitlements_xml(SettingsScope::Main),
-            Some("main_exe")
+            macho_settings.info_plist_data(SettingsScope::Main),
+            Some(b"main_exe".as_ref())
         );
         assert_eq!(
-            macho_settings.entitlements,
+            macho_settings.info_plist_data,
             [
-                (SettingsScope::Main, "main_exe".into()),
-                (SettingsScope::MultiArchIndex(0), "main_exe_index_0".into()),
+                (SettingsScope::Main, b"main_exe".to_vec()),
+                (
+                    SettingsScope::MultiArchIndex(0),
+                    b"main_exe_index_0".to_vec()
+                ),
                 (
                     SettingsScope::MultiArchCpuType(CPU_TYPE_X86_64),
-                    "main_exe_x86_64".into()
+                    b"main_exe_x86_64".to_vec()
                 ),
             ]
             .iter()
             .cloned()
-            .collect::<BTreeMap<SettingsScope, String>>()
+            .collect::<BTreeMap<SettingsScope, Vec<u8>>>()
         );
     }
 
     #[test]
     fn as_nested_bundle_settings() {
         let mut main_settings = SigningSettings::default();
-        main_settings.set_entitlements_xml(SettingsScope::Main, "main");
-        main_settings.set_entitlements_xml(
+        main_settings.set_info_plist_data(SettingsScope::Main, b"main".to_vec());
+        main_settings.set_info_plist_data(
             SettingsScope::Path("Contents/MacOS/main".into()),
-            "main_exe",
+            b"main_exe".to_vec(),
         );
-        main_settings.set_entitlements_xml(
+        main_settings.set_info_plist_data(
             SettingsScope::Path("Contents/MacOS/nested.app".into()),
-            "bundle",
+            b"bundle".to_vec(),
         );
-        main_settings.set_entitlements_xml(
+        main_settings.set_info_plist_data(
             SettingsScope::PathMultiArchIndex("Contents/MacOS/nested.app".into(), 0),
-            "bundle_index_0",
+            b"bundle_index_0".to_vec(),
         );
-        main_settings.set_entitlements_xml(
+        main_settings.set_info_plist_data(
             SettingsScope::PathMultiArchCpuType(
                 "Contents/MacOS/nested.app".into(),
                 CPU_TYPE_X86_64,
             ),
-            "bundle_x86_64",
+            b"bundle_x86_64".to_vec(),
         );
-        main_settings.set_entitlements_xml(
+        main_settings.set_info_plist_data(
             SettingsScope::Path("Contents/MacOS/nested.app/Contents/MacOS/nested".into()),
-            "nested_main_exe",
+            b"nested_main_exe".to_vec(),
         );
-        main_settings.set_entitlements_xml(
+        main_settings.set_info_plist_data(
             SettingsScope::PathMultiArchIndex(
                 "Contents/MacOS/nested.app/Contents/MacOS/nested".into(),
                 0,
             ),
-            "nested_main_exe_index_0",
+            b"nested_main_exe_index_0".to_vec(),
         );
-        main_settings.set_entitlements_xml(
+        main_settings.set_info_plist_data(
             SettingsScope::PathMultiArchCpuType(
                 "Contents/MacOS/nested.app/Contents/MacOS/nested".into(),
                 CPU_TYPE_X86_64,
             ),
-            "nested_main_exe_x86_64",
+            b"nested_main_exe_x86_64".to_vec(),
         );
 
         let bundle_settings = main_settings.as_nested_bundle_settings("Contents/MacOS/nested.app");
         assert_eq!(
-            bundle_settings.entitlements_xml(SettingsScope::Main),
-            Some("bundle")
+            bundle_settings.info_plist_data(SettingsScope::Main),
+            Some(b"bundle".as_ref())
         );
         assert_eq!(
-            bundle_settings.entitlements_xml(SettingsScope::Path("Contents/MacOS/nested".into())),
-            Some("nested_main_exe")
+            bundle_settings.info_plist_data(SettingsScope::Path("Contents/MacOS/nested".into())),
+            Some(b"nested_main_exe".as_ref())
         );
         assert_eq!(
-            bundle_settings.entitlements,
+            bundle_settings.info_plist_data,
             [
-                (SettingsScope::Main, "bundle".into()),
-                (SettingsScope::MultiArchIndex(0), "bundle_index_0".into()),
+                (SettingsScope::Main, b"bundle".to_vec()),
+                (SettingsScope::MultiArchIndex(0), b"bundle_index_0".to_vec()),
                 (
                     SettingsScope::MultiArchCpuType(CPU_TYPE_X86_64),
-                    "bundle_x86_64".into()
+                    b"bundle_x86_64".to_vec()
                 ),
                 (
                     SettingsScope::Path("Contents/MacOS/nested".into()),
-                    "nested_main_exe".into()
+                    b"nested_main_exe".to_vec()
                 ),
                 (
                     SettingsScope::PathMultiArchIndex("Contents/MacOS/nested".into(), 0),
-                    "nested_main_exe_index_0".into()
+                    b"nested_main_exe_index_0".to_vec()
                 ),
                 (
                     SettingsScope::PathMultiArchCpuType(
                         "Contents/MacOS/nested".into(),
                         CPU_TYPE_X86_64
                     ),
-                    "nested_main_exe_x86_64".into()
+                    b"nested_main_exe_x86_64".to_vec()
                 ),
             ]
             .iter()
             .cloned()
-            .collect::<BTreeMap<SettingsScope, String>>()
+            .collect::<BTreeMap<SettingsScope, Vec<u8>>>()
         );
     }
+
+    #[test]
+    fn entitlements_handling() -> Result<(), AppleCodesignError> {
+        let mut settings = SigningSettings::default();
+        settings.set_entitlements_xml(SettingsScope::Main, ENTITLEMENTS_XML)?;
+
+        let s = settings.entitlements_xml(SettingsScope::Main)?;
+        assert_eq!(s, Some("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<!DOCTYPE plist PUBLIC \"-//Apple//DTD PLIST 1.0//EN\" \"http://www.apple.com/DTDs/PropertyList-1.0.dtd\">\n<plist version=\"1.0\">\n<dict>\n\t<key>application-identifier</key>\n\t<string>appid</string>\n\t<key>com.apple.developer.team-identifier</key>\n\t<string>ABCDEF</string>\n</dict>\n</plist>".into()));
+
+        Ok(())
+    }
 }
